import random
import string
from dataclasses import dataclass
from typing import List

import requests
from bs4 import BeautifulSoup

CAPTCHA_URL = "http://revivelink.com/qcap/Qaptcha.jquery.php"


@dataclass
class Link:
    platform: str
    url: str


def _generate_captcha_key() -> str:
    letters_and_digits = string.ascii_letters + string.digits
    return "".join(random.choice(letters_and_digits) for _ in range(32))


def _generate_sessionid(verify: bool = True) -> str:
    data = {"action": "qaptcha", "qaptcha_key": _generate_captcha_key()}
    response = requests.post(CAPTCHA_URL, data=data, verify=verify)
    return str(response.cookies["PHPSESSID"])


def _get_destination_url(url: str) -> str:
    splitted = url.split("/")
    splitted[-1] = f"slinks.php?R={splitted[-1]}"
    return "/".join(splitted)


def get_links(url: str, verify: bool = True) -> List[Link]:
    response = requests.get(
        _get_destination_url(url),
        allow_redirects=True,
        timeout=20,
        cookies={"PHPSESSID": _generate_sessionid(verify=verify)},
        verify=verify,
    )
    response.raise_for_status()

    soup_response = BeautifulSoup(response.content, "html.parser")

    result: List[Link] = []

    for row in soup_response.find_all("tr"):
        link = row.find("a")

        if link:
            tds = row.find_all("td")

            if len(tds) > 2:
                result.append(
                    Link(
                        url=link["href"],
                        platform="".join(tds[1].text.splitlines()).strip(),
                    )
                )

    return result



from urllib.request import Request

from vcr import VCR
from vcr_unittest import VCRTestCase


def custom_headers_check(new_request: Request, cassette_request: Request):
    headers1 = {k: v for k, v in new_request.headers.items() if k != "User-Agent"}
    headers2 = {k: v for k, v in cassette_request.headers.items() if k != "User-Agent"}
    assert headers1 == headers2, f"{headers1} != {headers2}"


class RevivelinkBypassTest(VCRTestCase):
    record_mode = "once"

    def _get_vcr(self, **kwargs) -> VCR:
        vcr = super()._get_vcr(**kwargs)
        vcr.register_matcher("headers", custom_headers_check)
        vcr.match_on = vcr.matchers.keys()
        vcr.record_mode = self.record_mode
        return vcr

    def _get_cassette_name(self) -> str:
        return f"{self.__class__.__name__}.{self._testMethodName}.yaml"



import random

from revivelink_bypass import get_links
from revivelink_bypass.tests.conftest import RevivelinkBypassTest


class TestGetUrl(RevivelinkBypassTest):
    def setUp(self):
        super().setUp()
        random.seed(0)

    def test_get_urls(self) -> None:
        links = get_links("http://revivelink.com/BYPASS")

        self.assertEqual(2, len(links))

        self.assertEqual("Google", links[0].platform)
        self.assertEqual("https://www.google.com/", links[0].url)
        self.assertEqual("Bing", links[1].platform)
        self.assertEqual("https://www.bing.com/", links[1].url)



import pathlib

from setuptools import find_packages, setup

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text(encoding="UTF-8")

setup(
    name="revivelink_bypass",
    version="0.0.2",
    description="",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/florentclarret/revivelink-bypass",
    project_urls={
        "Changelog": "https://github.com/FlorentClarret/revivelink-bypass/blob/main/CHANGELOG.md"
    },
    author="Florent Clarret",
    author_email="florent.clarret@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    packages=find_packages(),
    include_package_data=True,
    package_data={
        "revivelink_bypass": ["py.typed"],
    },
    install_requires=[
        "beautifulsoup4==4.11.1",
        "requests==2.28.1",
    ],
)
